@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -263,6 +263,9 @@ e_text_to_html_full (const char *input, unsigned int flags, guint32 color)
 			    !g_ascii_strncasecmp ((gchar *) cur, "https://", 8) ||
 			    !g_ascii_strncasecmp ((gchar *) cur, "ftp://", 6) ||
 			    !g_ascii_strncasecmp ((gchar *) cur, "nntp://", 7) ||
+			    !g_ascii_strncasecmp ((gchar *) cur, "callto:", 7) ||
+			    !g_ascii_strncasecmp ((gchar *) cur, "h323:", 5) ||
+			    !g_ascii_strncasecmp ((gchar *) cur, "sip:", 4) ||
 			    !g_ascii_strncasecmp ((gchar *) cur, "mailto:", 7) ||
 			    !g_ascii_strncasecmp ((gchar *) cur, "news:", 5)) {
 				tmpurl = url_extract (&cur, TRUE);
